@@ -53,23 +53,22 @@ PyObject *pyfastx_fasta_new(PyTypeObject *type, PyObject *args, PyObject *kwargs
 	int full_name = 0;
 
 	//fasta file path
-	char *file_name;
-
-	Py_ssize_t file_len;
+	PyObject *file_obj;
+	PyObject *index_obj = NULL;
 
 	//key function for seperating name
-	PyObject *key_func = Py_None;
+	PyObject *key_func = NULL;
 
 	pyfastx_Fasta *obj;
 
 	//paramters for fasta object construction
-	static char* keywords[] = {"file_name", "uppercase", "build_index", "full_index", "full_name", "memory_index", "key_func", NULL};
+	static char* keywords[] = {"file_name", "index_file", "uppercase", "build_index", "full_index", "full_name", "memory_index", "key_func", NULL};
 	
-	if(!PyArg_ParseTupleAndKeywords(args, kwargs, "s#|iiiiiO", keywords, &file_name, &file_len, &uppercase, &build_index, &full_index, &full_name, &memory_index, &key_func)){
+	if(!PyArg_ParseTupleAndKeywords(args, kwargs, "O|OiiiiiO", keywords, &file_obj, &index_obj, &uppercase, &build_index, &full_index, &full_name, &memory_index, &key_func)){
 		return NULL;
 	}
 
-	if ((key_func != Py_None) && !PyCallable_Check(key_func)) {
+	if ((key_func) && !PyCallable_Check(key_func)) {
 		PyErr_SetString(PyExc_TypeError, "key_func must be a callable function");
 		return NULL;
 	}
@@ -82,8 +81,8 @@ PyObject *pyfastx_fasta_new(PyTypeObject *type, PyObject *args, PyObject *kwargs
 		return NULL;
 	}*/
 
-	if (!file_exists(file_name)) {
-		PyErr_Format(PyExc_FileExistsError, "the input fasta file %s does not exists", file_name);
+	if (!file_exists(file_obj)) {
+		PyErr_Format(PyExc_FileExistsError, "the input fasta file %U does not exists", file_obj);
 		return NULL;
 	}
 
@@ -92,22 +91,21 @@ PyObject *pyfastx_fasta_new(PyTypeObject *type, PyObject *args, PyObject *kwargs
 	if (!obj) return NULL;
 
 	//initial sequence file name
-	obj->file_name = (char *)malloc((int)file_len + 1);
-	strcpy(obj->file_name, file_name);
+	obj->file_obj = Py_NewRef(file_obj);
 
 	obj->uppercase = uppercase;
 	obj->has_index = build_index;
 
 	//create index
 
-	obj->index = pyfastx_init_index((PyObject *)obj, obj->file_name, (int)file_len, uppercase, full_name, memory_index, key_func);
+	obj->index = pyfastx_init_index((PyObject *)obj, file_obj, index_obj, uppercase, full_name, memory_index, key_func);
 	
 	//iter function
 	obj->func = pyfastx_index_next_null;
 
 	//check is correct fasta format
 	if (!fasta_validator(obj->index->gzfd)) {
-		PyErr_Format(PyExc_RuntimeError, "%s is not plain or gzip compressed fasta formatted file", file_name);
+		PyErr_Format(PyExc_RuntimeError, "%U is not plain or gzip compressed fasta formatted file", file_obj);
 		return NULL;
 	}
 
@@ -130,17 +128,17 @@ PyObject *pyfastx_fasta_new(PyTypeObject *type, PyObject *args, PyObject *kwargs
 }
 
 void pyfastx_fasta_dealloc(pyfastx_Fasta *self){
-	free(self->file_name);
-
+	//free(self->file_name);
 	pyfastx_index_free(self->index);
+	Py_DECREF(self->file_obj);
 	Py_TYPE(self)->tp_free((PyObject *)self);
 }
 
 PyObject *pyfastx_fasta_repr(pyfastx_Fasta *self) {
 	if (self->has_index) {
-		return PyUnicode_FromFormat("<Fasta> %s contains %ld sequences", self->file_name, self->seq_counts);
+		return PyUnicode_FromFormat("<Fasta> %U contains %zd sequences", self->file_obj, self->seq_counts);
 	} else {
-		return PyUnicode_FromFormat("<Fasta> %s", self->file_name);
+		return PyUnicode_FromFormat("<Fasta> %U", self->file_obj);
 	}
 }
 
@@ -859,11 +857,11 @@ void pyfastx_fasta_calc_composition(pyfastx_Fasta *self) {
 
 	//ascii char statistics
 	Py_ssize_t seq_comp[128] = {0};
-	Py_ssize_t fa_comp[26] = {0};
+	Py_ssize_t fa_comp[128] = {0};
 
 	Py_ssize_t i;
-	
-	
+	Py_ssize_t seqid = 0;
+
 	sql = "SELECT * FROM comp LIMIT 1";
 
 	PYFASTX_SQLITE_CALL(
@@ -880,7 +878,7 @@ void pyfastx_fasta_calc_composition(pyfastx_Fasta *self) {
 	sql = "PRAGMA synchronous=OFF;BEGIN TRANSACTION;";
 	PYFASTX_SQLITE_CALL(sqlite3_exec(self->index->index_db, sql, NULL, NULL, NULL));
 
-	sql = "INSERT INTO comp VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?);";
+	sql = "INSERT INTO comp VALUES (?,?,?,?);";
 	PYFASTX_SQLITE_CALL(sqlite3_prepare_v2(self->index->index_db, sql, -1, &stmt, NULL));
 	
 	gzrewind(self->index->gzfd);
@@ -890,51 +888,60 @@ void pyfastx_fasta_calc_composition(pyfastx_Fasta *self) {
 	
 	while (ks_getuntil(ks, '\n', &line, 0) >= 0) {
 		if (line.s[0] == 62) {
-			if (sum_array(seq_comp, 128) > 0) {
-				j = 0;
-				sqlite3_bind_null(stmt, ++j);
-				for (c = 65; c <= 90; c++) {
-					sqlite3_bind_int64(stmt, ++j, seq_comp[c]+seq_comp[c+32]);
-					fa_comp[c-65] += seq_comp[c]+seq_comp[c+32];
+			if (seqid > 0) {
+				for (j = 0; j < 128; ++j) {
+					if (seq_comp[j] > 0) {
+						sqlite3_bind_null(stmt, 1);
+						sqlite3_bind_int64(stmt, 2, seqid);
+						sqlite3_bind_int(stmt, 3, j);
+						sqlite3_bind_int64(stmt, 4, seq_comp[j]);
+						sqlite3_step(stmt);
+						sqlite3_reset(stmt);
+						fa_comp[j] += seq_comp[j];
+					}
 				}
-				sqlite3_step(stmt);
-				sqlite3_reset(stmt);
 			}
+
 			memset(seq_comp, 0, sizeof(seq_comp));
+			seqid++;
 			continue;
 		}
 
-		for (i = 0; i < line.l; i++) {
+		for (i = 0; i < line.l; ++i) {
 			c = line.s[i];
-			if (c == 10 || c == 13) {
-				continue;
-			}
+
 			++seq_comp[c];
 		}
 	}
 
-	if (sum_array(seq_comp, 128) > 0) {
-		j = 0;
-		sqlite3_bind_null(stmt, ++j);
-		for (c = 65; c <= 90; c++) {
-			sqlite3_bind_int64(stmt, ++j, seq_comp[c]+seq_comp[c+32]);
-			fa_comp[c-65] += seq_comp[c]+seq_comp[c+32];
+	//write the last sequence
+	for (j = 0; j < 128; ++j) {
+		if (seq_comp[j] > 0) {
+			sqlite3_bind_null(stmt, 1);
+			sqlite3_bind_int64(stmt, 2, seqid);
+			sqlite3_bind_int(stmt, 3, j);
+			sqlite3_bind_int64(stmt, 4, seq_comp[j]);
+			sqlite3_step(stmt);
+			sqlite3_reset(stmt);
+			fa_comp[j] += seq_comp[j];
 		}
+	}
+
+	//write total composition to db
+	for (j = 0; j < 128; ++j) {
+		sqlite3_bind_null(stmt, 1);
+		sqlite3_bind_int64(stmt, 2, 0);
+		sqlite3_bind_int(stmt, 3, j);
+		sqlite3_bind_int64(stmt, 4, fa_comp[j]);
 		sqlite3_step(stmt);
 		sqlite3_reset(stmt);
 	}
 
-	//write total bases to db
-	sqlite3_bind_null(stmt, 1);
-	for (j = 0; j < 26; j++) {
-		sqlite3_bind_int64(stmt, j+2, fa_comp[j]);
-	}
-	sqlite3_step(stmt);
 	sqlite3_finalize(stmt);
 	sqlite3_exec(self->index->index_db, "COMMIT;", NULL, NULL, NULL);
 
 	Py_END_ALLOW_THREADS
-	
+
 	ks_destroy(ks);
 	free(line.s);
 }
@@ -943,28 +950,57 @@ PyObject *pyfastx_fasta_gc_content(pyfastx_Fasta *self, void* closure) {
 	int ret;
 	const char *sql;
 	sqlite3_stmt *stmt;
-	Py_ssize_t a, c, g, t;
+
+	int l;
+	Py_ssize_t n;
+	Py_ssize_t a = 0;
+	Py_ssize_t c = 0;
+	Py_ssize_t g = 0;
+	Py_ssize_t t = 0;
 
 	pyfastx_fasta_calc_composition(self);
-	sql = "SELECT a, c, g, t FROM comp ORDER BY ID DESC LIMIT 1";
+	sql = "SELECT * FROM comp WHERE seqid=0";
 	
 	PYFASTX_SQLITE_CALL(
 		sqlite3_prepare_v2(self->index->index_db, sql, -1, &stmt, NULL);
 		ret = sqlite3_step(stmt);
 	);
 
-	if (ret == SQLITE_ROW) {
+	while (ret == SQLITE_ROW) {
 		PYFASTX_SQLITE_CALL(
-			a = sqlite3_column_int64(stmt, 0);
-			c = sqlite3_column_int64(stmt, 1);
-			g = sqlite3_column_int64(stmt, 2);
-			t = sqlite3_column_int64(stmt, 3);
-			sqlite3_finalize(stmt);
+			l = sqlite3_column_int(stmt, 2);
+			n = sqlite3_column_int64(stmt, 3);
+			ret = sqlite3_step(stmt);
 		);
+
+		switch (l) {
+			case 65:
+			case 97:
+				a += n;
+				break;
+
+			case 67:
+			case 99:
+				c += n;
+				break;
+
+			case 71:
+			case 103:
+				g += n;
+				break;
+
+			case 84:
+			case 116:
+				t += n;
+				break;
+		}
+	}
+
+	PYFASTX_SQLITE_CALL(sqlite3_finalize(stmt));
+
+	if (a + c + g + t > 0) {
 		return Py_BuildValue("f", (float)(g+c)/(a+c+g+t)*100);
-	
 	} else {
-		PYFASTX_SQLITE_CALL(sqlite3_finalize(stmt));
 		PyErr_SetString(PyExc_RuntimeError, "could not calculate gc content");
 		return NULL;
 	}
@@ -974,77 +1010,101 @@ PyObject *pyfastx_fasta_gc_skew(pyfastx_Fasta *self, void* closure) {
 	int ret;
 	const char *sql;
 	sqlite3_stmt *stmt;
-	Py_ssize_t c, g;
+
+	int l;
+	Py_ssize_t n;
+	Py_ssize_t c = 0;
+	Py_ssize_t g = 0;
 
 	pyfastx_fasta_calc_composition(self);
-	sql = "SELECT c, g FROM comp ORDER BY ID DESC LIMIT 1";
+	sql = "SELECT * FROM comp WHERE seqid=0";
+	
 	PYFASTX_SQLITE_CALL(
 		sqlite3_prepare_v2(self->index->index_db, sql, -1, &stmt, NULL);
 		ret = sqlite3_step(stmt);
 	);
-	
-	if (ret == SQLITE_ROW) {
+
+	while (ret == SQLITE_ROW) {
 		PYFASTX_SQLITE_CALL(
-			c = sqlite3_column_int64(stmt, 0);
-			g = sqlite3_column_int64(stmt, 1);
-			sqlite3_finalize(stmt);
+			l = sqlite3_column_int(stmt, 2);
+			n = sqlite3_column_int64(stmt, 3);
+			ret = sqlite3_step(stmt);
 		);
+
+		switch (l) {
+			case 67:
+			case 99:
+				c += n;
+				break;
+
+			case 71:
+			case 103:
+				g += n;
+				break;
+		}
+	}
+
+	PYFASTX_SQLITE_CALL(sqlite3_finalize(stmt));
+
+	if (c + g > 0) {
 		return Py_BuildValue("f", (float)(g-c)/(g+c));
 	} else {
-		PYFASTX_SQLITE_CALL(sqlite3_finalize(stmt));
 		PyErr_SetString(PyExc_RuntimeError, "could not calculate gc skew");
 		return NULL;
 	}
 }
 
 PyObject *pyfastx_fasta_composition(pyfastx_Fasta *self, void* closure) {
-	int i;
+	int l;
 	int ret;
 	const char *sql;
 
 	sqlite3_stmt *stmt;
 	
-	Py_ssize_t c;
+	Py_ssize_t n;
+
 	PyObject *d;
-	PyObject *base;
-	PyObject *count;
+	PyObject *b;
+	PyObject *c;
 
 	pyfastx_fasta_calc_composition(self);
 
 	//the last row store the sum of the each base
-	sql = "SELECT * FROM comp ORDER BY ID DESC LIMIT 1";
+	sql = "SELECT * FROM comp WHERE seqid=0";
 	PYFASTX_SQLITE_CALL(
 		sqlite3_prepare_v2(self->index->index_db, sql, -1, &stmt, NULL);
 		ret = sqlite3_step(stmt);
 	);
-	
-	if (ret == SQLITE_ROW) {
-		d = PyDict_New();
-		for (i = 1; i < 27; i++) {
-			PYFASTX_SQLITE_CALL(c = sqlite3_column_int64(stmt, i));
-			if (c > 0) {
-				base = Py_BuildValue("C", i+64);
-				count = Py_BuildValue("n", c);
-				PyDict_SetItem(d, base, count);
-				Py_DECREF(base);
-				Py_DECREF(count);
-			}
+
+	d = PyDict_New();
+
+	while (ret == SQLITE_ROW) {
+		PYFASTX_SQLITE_CALL(
+			l = sqlite3_column_int(stmt, 2);
+			n = sqlite3_column_int64(stmt, 3);
+			ret = sqlite3_step(stmt);
+		);
+
+		if (n > 0 && l != 13) {
+			b = Py_BuildValue("C", l);
+			c = Py_BuildValue("n", n);
+			PyDict_SetItem(d, b, c);
+			Py_DECREF(b);
+			Py_DECREF(c);
 		}
-		PYFASTX_SQLITE_CALL(sqlite3_finalize(stmt));
-		return d;
-		
-	} else {
-		PYFASTX_SQLITE_CALL(sqlite3_finalize(stmt));
-		PyErr_SetString(PyExc_RuntimeError, "could not get composition");
-		return NULL;
 	}
+
+	PYFASTX_SQLITE_CALL(sqlite3_finalize(stmt));
+
+	return d;
 }
 
 //support for guess sequence type according to IUPAC codes
 //https://www.bioinformatics.org/sms/iupac.html
 PyObject *pyfastx_fasta_guess_type(pyfastx_Fasta *self, void* closure) {
+	int l;
 	int ret;
-	int i, j;
+	int i;
 
 	char *alphabets;
 	char *retval;
@@ -1052,38 +1112,39 @@ PyObject *pyfastx_fasta_guess_type(pyfastx_Fasta *self, void* closure) {
 
 	sqlite3_stmt *stmt;
 
-	Py_ssize_t c;
+	Py_ssize_t n;
 
 	pyfastx_fasta_calc_composition(self);
 
-	sql = "SELECT * FROM comp ORDER BY ID DESC LIMIT 1";
+	sql = "SELECT * FROM comp WHERE seqid=0";
 	PYFASTX_SQLITE_CALL(
 		sqlite3_prepare_v2(self->index->index_db, sql, -1, &stmt, NULL);
 		ret = sqlite3_step(stmt);
 	);
 
-	if (ret != SQLITE_ROW) {
-		PYFASTX_SQLITE_CALL(sqlite3_finalize(stmt));
-		PyErr_SetString(PyExc_RuntimeError, "could not get sequence type");
-		return NULL;
-	}
+	i = 0;
+	alphabets = (char *)malloc(128);
 
-	alphabets = (char *)malloc(26);
-	j = 0;
-	for (i = 1; i < 27; i++) {
-		PYFASTX_SQLITE_CALL(c = sqlite3_column_int64(stmt, i));
-		if (c > 0) {
-			alphabets[j++] = i+64;
+	while (ret == SQLITE_ROW) {
+		PYFASTX_SQLITE_CALL(
+			l = sqlite3_column_int(stmt, 2);
+			n = sqlite3_column_int64(stmt, 3);
+			ret = sqlite3_step(stmt);
+		);
+
+		if (l > 32 && l < 127 && n > 0) {
+			alphabets[i++] = l;
 		}
 	}
-	alphabets[j] = '\0';
+
+	alphabets[i] = '\0';
 	PYFASTX_SQLITE_CALL(sqlite3_finalize(stmt));
 
-	if (is_subset("ACGTN", alphabets) || is_subset("ABCDGHKMNRSTVWY-", alphabets)) {
+	if (is_subset("ACGTNacgtn", alphabets) || is_subset("abcdghkmnrstvwyABCDGHKMNRSTVWY*-", alphabets)) {
 		retval = "DNA";
-	} else if (is_subset("ACGUN", alphabets) || is_subset("ABCDGHKMNRSUVWY-", alphabets)) {
+	} else if (is_subset("ACGUNacgun", alphabets) || is_subset("abcdghkmnrsuvwyABCDGHKMNRSUVWY*-", alphabets)) {
 		retval = "RNA";
-	} else if (is_subset("ACDEFGHIKLMNPQRSTVWY*-", alphabets)) {
+	} else if (is_subset("acdefghiklmnpqrstvwyACDEFGHIKLMNPQRSTVWY*-", alphabets)) {
 		retval = "protein";
 	} else {
 		retval = "unknown";
@@ -1106,7 +1167,7 @@ static PyGetSetDef pyfastx_fasta_getsets[] = {
 };
 
 static PyMemberDef pyfastx_fasta_members[] = {
-	{"file_name", T_STRING, offsetof(pyfastx_Fasta, file_name), READONLY},
+	{"file_name", T_OBJECT, offsetof(pyfastx_Fasta, file_obj), READONLY},
 	{"size", T_PYSSIZET, offsetof(pyfastx_Fasta, seq_length), READONLY},
 	{NULL}
 };
