@@ -53,7 +53,7 @@ typedef struct {
 	PyObject_HEAD
 
 	//fastq file name
-	char* file_name;
+	PyObject* file_obj;
 
 	//index file
 	char* index_file;
@@ -103,9 +103,6 @@ typedef struct {
 
 extern PyTypeObject pyfastx_FastqType;
 
-void pyfastx_fastq_dealloc(pyfastx_Fastq *self);
-void pyfastx_fastq_create_index(pyfastx_Fastq *self);
 void pyfastx_fastq_calc_composition(pyfastx_Fastq *self);
-PyObject *pyfastx_fastq_new(PyTypeObject *type, PyObject *args, PyObject *kwargs);
 
 #endif
\ No newline at end of file
