@@ -12,35 +12,52 @@ create an index
 @param uppercase, uppercase sequence
 @param uppercase
 */
-pyfastx_Index* pyfastx_init_index(PyObject *obj, char* file_name, int file_len, int uppercase, int full_name, int memory_index, PyObject* key_func){
+pyfastx_Index* pyfastx_init_index(PyObject *obj, PyObject* file_obj, PyObject* index_obj, int uppercase, int full_name, int memory_index, PyObject* key_func){
 	pyfastx_Index* index;
 
+	char *index_file;
+	Py_ssize_t index_len;
+
 	index = (pyfastx_Index *)malloc(sizeof(pyfastx_Index));
 	index->uppercase = uppercase;
 
 	//key function
-	index->key_func = key_func;
+	if (key_func) {
+		index->key_func = Py_NewRef(key_func);
+	} else {
+		index->key_func = NULL;
+	}
 
 	//full name
 	index->full_name = full_name;
 
 	//check input file is gzip or not
-	index->gzip_format = is_gzip_format(file_name);
+	index->gzip_format = is_gzip_format(file_obj);
 
 	//initial kseqs
-	index->gzfd = gzopen(file_name, "rb");
+	index->gzfd = pyfastx_gzip_open(file_obj, "rb");
 	index->kseqs = kseq_init(index->gzfd);
 
 	//create index file or memory index
 	if (memory_index) {
-		index->index_file = ":memory:";
+		index->index_file = (char *)malloc(sizeof(char)*9);
+		strcpy(index->index_file, ":memory:");
 	} else {
-		index->index_file = (char *)malloc(file_len + 5);
-		strcpy(index->index_file, file_name);
-		strcat(index->index_file, ".fxi");
+		if (index_obj) {
+			index_file = (char *)PyUnicode_AsUTF8AndSize(index_obj, &index_len);
+			index->index_file = (char *)malloc(index_len);
+			memcpy(index->index_file, index_file, index_len);
+			index->index_file[index_len] = '\0';
+		} else {
+			index_file = (char *)PyUnicode_AsUTF8AndSize(file_obj, &index_len);
+			index_len += 5;
+			index->index_file = (char *)malloc(index_len);
+			strcpy(index->index_file, index_file);
+			strcat(index->index_file, ".fxi");
+		}
 	}
 
-	index->fd = fopen(file_name, "rb");
+	index->fd = _Py_fopen_obj(file_obj, "rb");
 
 	index->index_db = 0;
 
@@ -149,7 +166,7 @@ void pyfastx_create_index(pyfastx_Index *self){
 	PYFASTX_SQLITE_CALL(ret = sqlite3_open(self->index_file, &self->index_db));
 	
 	if (ret != SQLITE_OK) {
-		PyErr_Format(PyExc_ConnectionError, "Can not open index file %s", self->index_file);
+		PyErr_Format(PyExc_ConnectionError, "Could not open index file %s", self->index_file);
 		return;
 	}
 	
@@ -175,33 +192,10 @@ void pyfastx_create_index(pyfastx_Index *self){
 			l50 INTEGER --N50 seq count \n \
 		); \
 		CREATE TABLE comp ( \
-			ID INTEGER PRIMARY KEY, \
-			a INTEGER, \
-			b INTEGER, \
-			c INTEGER, \
-			d INTEGER, \
-			e INTEGER, \
-			f INTEGER, \
-			g INTEGER, \
-			h INTEGER, \
-			i INTEGER, \
-			j INTEGER, \
-			k INTEGER, \
-			l INTEGER, \
-			m INTEGER, \
-			n INTEGER, \
-			o INTEGER, \
-			p INTEGER, \
-			q INTEGER, \
-			r INTEGER, \
-			s INTEGER, \
-			t INTEGER, \
-			u INTEGER, \
-			v INTEGER, \
-			w INTEGER, \
-			x INTEGER, \
-			y INTEGER, \
-			z INTEGER \
+			ID INTEGER PRIMARY KEY, --comp identifier\n \
+			seqid INTEGER, --seq id\n \
+			abc INTEGER, --seq letter\n \
+			num INTEGER -- letter count\n \
 		); \
 		CREATE TABLE gzindex ( \
 			ID INTEGER PRIMARY KEY, \
@@ -280,7 +274,7 @@ void pyfastx_create_index(pyfastx_Index *self){
 			//remove > sign at the start position
 			header_pos = line.s + 1;
 
-			if (self->key_func == Py_None) {
+			if (self->key_func == NULL) {
 				if (self->full_name) {
 					chrom.l = desc_len;
 					memcpy(chrom.s, header_pos, chrom.l);
@@ -384,7 +378,7 @@ void pyfastx_create_index(pyfastx_Index *self){
 		if (strcmp(self->index_file, ":memory:") == 0) {
 			zran_build_index(self->gzip_index, 0, 0);
 		} else {
-			pyfastx_build_gzip_index(self->index_file, self->gzip_index, self->index_db);
+			pyfastx_build_gzip_index(self->gzip_index, self->index_db);
 		}
 	}
 }
@@ -413,16 +407,21 @@ void pyfastx_load_index(pyfastx_Index *self){
 	}
 
 	if (self->gzip_format) {
-		pyfastx_load_gzip_index(self->index_file, self->gzip_index, self->index_db);
+		pyfastx_load_gzip_index(self->gzip_index, self->index_db);
 	}
 }
 
 void pyfastx_build_index(pyfastx_Index *self){
-	if (file_exists(self->index_file)) {
+	PyObject *index_obj;
+	index_obj = PyUnicode_FromString(self->index_file);
+
+	if (file_exists(index_obj)) {
 		pyfastx_load_index(self);
 	} else {
 		pyfastx_create_index(self);
 	}
+
+	Py_DECREF(index_obj);
 }
 
 void pyfastx_index_free(pyfastx_Index *self){
@@ -586,7 +585,7 @@ PyObject *pyfastx_index_get_seq_by_id(pyfastx_Index *self, Py_ssize_t chrom){
 		);
 
 		return (PyObject *)obj;
-	} else { 
+	} else {
 		PYFASTX_SQLITE_CALL(sqlite3_reset(self->uid_stmt));
 		PyErr_SetString(PyExc_IndexError, "Index Error");
 		return NULL;
